#include "tm.h"


#include <fstream>

void TM::Allocate(int vsN, int tsN) {

	vertsN = vsN;
	verts = new V3[vertsN];
	cols = new V3[vertsN];
	trisN = tsN;
	tris = new unsigned[3*trisN];

}

void TM::SetToBox(V3 O, V3 dims, V3 color) {

	vertsN = 8;
	trisN = 6 * 2;
	Allocate(vertsN, trisN);

	for (int vi = 0; vi < vertsN/2; vi++) {
		cols[vi] = color;
		cols[vi + vertsN / 2] = color*0.5f;
	}

	int vi = 0;
	verts[vi++] = O + V3(-dims[0] / 2.0f, +dims[1] / 2.0f, +dims[2] / 2.0f);
	verts[vi++] = O + V3(-dims[0] / 2.0f, -dims[1] / 2.0f, +dims[2] / 2.0f);
	verts[vi++] = O + V3(+dims[0] / 2.0f, -dims[1] / 2.0f, +dims[2] / 2.0f);
	verts[vi++] = O + V3(+dims[0] / 2.0f, +dims[1] / 2.0f, +dims[2] / 2.0f);

	verts[vi++] = O + V3(-dims[0] / 2.0f, +dims[1] / 2.0f, -dims[2] / 2.0f);
	verts[vi++] = O + V3(-dims[0] / 2.0f, -dims[1] / 2.0f, -dims[2] / 2.0f);
	verts[vi++] = O + V3(+dims[0] / 2.0f, -dims[1] / 2.0f, -dims[2] / 2.0f);
	verts[vi++] = O + V3(+dims[0] / 2.0f, +dims[1] / 2.0f, -dims[2] / 2.0f);

	trisN = 0;
	int tri = 0;

	tris[tri * 3 + 0] = 0;
	tris[tri * 3 + 1] = 1;
	tris[tri * 3 + 2] = 2;
	tri++;
	tris[tri * 3 + 0] = 2;
	tris[tri * 3 + 1] = 3;
	tris[tri * 3 + 2] = 0;
	tri++;
	trisN += 2;

	tris[tri * 3 + 0] = 3;
	tris[tri * 3 + 1] = 2;
	tris[tri * 3 + 2] = 6;
	tri++;
	tris[tri * 3 + 0] = 6;
	tris[tri * 3 + 1] = 7;
	tris[tri * 3 + 2] = 3;
	tri++;
	trisN += 2;

	tris[tri * 3 + 0] = 7;
	tris[tri * 3 + 1] = 6;
	tris[tri * 3 + 2] = 5;
	tri++;
	tris[tri * 3 + 0] = 5;
	tris[tri * 3 + 1] = 4;
	tris[tri * 3 + 2] = 7;
	tri++;
	trisN += 2;

	tris[tri * 3 + 0] = 4;
	tris[tri * 3 + 1] = 5;
	tris[tri * 3 + 2] = 1;
	tri++;
	tris[tri * 3 + 0] = 1;
	tris[tri * 3 + 1] = 0;
	tris[tri * 3 + 2] = 4;
	tri++;
	trisN += 2;


	tris[tri * 3 + 0] = 5;
	tris[tri * 3 + 1] = 6;
	tris[tri * 3 + 2] = 2;
	tri++;
	tris[tri * 3 + 0] = 2;
	tris[tri * 3 + 1] = 1;
	tris[tri * 3 + 2] = 5;
	tri++;
	trisN += 2;

	tris[tri * 3 + 0] = 0;
	tris[tri * 3 + 1] = 3;
	tris[tri * 3 + 2] = 7;
	tri++;
	tris[tri * 3 + 0] = 7;
	tris[tri * 3 + 1] = 4;
	tris[tri * 3 + 2] = 0;
	tri++;
	trisN += 2;





}

void TM::RenderPoints(PPC *ppc, FrameBuffer *fb) {

	for (int vi = 0; vi < vertsN; vi++) {
		V3 projV;
		if (!ppc->Project(verts[vi], projV))
			continue;
		unsigned int vcolor = cols[vi].GetColor();
		fb->DrawCircle((int) projV[0], (int) projV[1], 5, vcolor);
	}

}


void TM::RenderWireframe(PPC *ppc, FrameBuffer *fb) {

	for (int tri = 0; tri < trisN; tri++) {
		for (int ei = 0; ei < 3; ei++) {
			int ei1 = (ei + 1) % 3;
			V3 V0 = verts[tris[3 * tri + ei]];
			V3 V1 = verts[tris[3 * tri + ei1]];
			V3 c0 = cols[tris[3 * tri + ei]];
			V3 c1 = cols[tris[3 * tri + ei1]];
			fb->Draw3DSegment(V0, V1, c0, c1, ppc);
		}
	}

}

V3 TM::GetCenterOfMass() {

	V3 ret(0.0f, 0.0f, 0.0f);
	for (int vi = 0; vi < vertsN; vi++) {
		ret = ret + verts[vi];
	}
	ret = ret / (float)vertsN;
	return ret;

}


void TM::LoadBin(char *fname) {

	ifstream ifs(fname, ios::binary);
	if (ifs.fail()) {
		cerr << "INFO: cannot open file: " << fname << endl;
		return;
	}

	ifs.read((char*)&vertsN, sizeof(int));
	char yn;
	ifs.read(&yn, 1); // always xyz
	if (yn != 'y') {
		cerr << "INTERNAL ERROR: there should always be vertex xyz data" << endl;
		return;
	}
	if (verts)
		delete verts;
	verts = new V3[vertsN];

	ifs.read(&yn, 1); // cols 3 floats
	if (cols)
		delete cols;
	cols = 0;
	if (yn == 'y') {
		cols = new V3[vertsN];
	}

	ifs.read(&yn, 1); // normals 3 floats
	V3 *normals = 0; // don't have normals for now
	if (normals)
		delete normals;
	normals = 0;
	if (yn == 'y') {
		normals = new V3[vertsN];
	}

	ifs.read(&yn, 1); // texture coordinates 2 floats
	float *tcs = 0; // don't have texture coordinates for now
	if (tcs)
		delete tcs;
	tcs = 0;
	if (yn == 'y') {
		tcs = new float[vertsN * 2];
	}

	ifs.read((char*)verts, vertsN * 3 * sizeof(float)); // load verts

	if (cols) {
		ifs.read((char*)cols, vertsN * 3 * sizeof(float)); // load cols
	}

	if (normals)
		ifs.read((char*)normals, vertsN * 3 * sizeof(float)); // load normals

	if (tcs)
		ifs.read((char*)tcs, vertsN * 2 * sizeof(float)); // load texture coordinates

	ifs.read((char*)&trisN, sizeof(int));
	if (tris)
		delete tris;
	tris = new unsigned int[trisN * 3];
	ifs.read((char*)tris, trisN * 3 * sizeof(unsigned int)); // read tiangles

	ifs.close();

	cerr << "INFO: loaded " << vertsN << " verts, " << trisN << " tris from " << endl << "      " << fname << endl;
	cerr << "      xyz " << ((cols) ? "rgb " : "") << ((normals) ? "nxnynz " : "") << ((tcs) ? "tcstct " : "") << endl;

	delete[]tcs;

}



void TM::RotateAboutAxis(V3 O, V3 a, float theta) {

	for (int vi = 0; vi < vertsN; vi++) {
		verts[vi] = verts[vi].RotatePointAboutAxis(O, a, theta);
	}

}
